define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/array",
    "thornberry/app/Component/_EditorComponent",
    "thornberry/app/Component/_TableFieldEditorMixin",
    "thornberry/app/Component/_DataFormMixin",
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",
    "dojo/text!./Templates/CloseFinancialPeriodEditor.html",
    "dojo/dom-style",
    "dojo/dom-class",
    "dojo/dom-construct",
    "dojo/on",
    "dojo/aspect",
    "dojo/when",
    "dojo/store/Memory",
    "thornberry/app/utils",
    "thornberry/app/Component/ScoreCard",
    "thornberry/app/Component/BillingManagement/ReportItem",
    "thornberry/PresetManager",
    "dijit/form/Button"
],function(
    declare,
    lang,
    arrayUtils,
    _EditorComponent,
    _TableFieldEditorMixin,
    _DataFormMixin,
    _TemplatedMixin,
    _WidgetsInTemplateMixin,
    template,
    domStyle,
    domClass,
    domConstruct,
    on,
    aspect,
    when,
    Memory,
    utils,
    ScoreCard,
    ReportItem,
    PresetManager
) {
    return declare([_EditorComponent,_TableFieldEditorMixin,_DataFormMixin,_TemplatedMixin,_WidgetsInTemplateMixin],{
        templateString: template,

        financialPeriod: null,
        reportItems: null,
        id: null,

        constructor: function() {
            var t = this;
            t.reportItems = [];
        },

        getInitialProps: function() {
            return lang.mixin(this.inherited(arguments),{
                settings: null,
                financialPeriod: null,
                financialPeriods: null,
                financialReports: null,
                onPushView: null,
                onGenerateReport: null
            });
        },

        buildRendering: function() {
            var t = this;
            t.inherited(arguments);

            t.setTable("LocationType",[
                { id: 1, name: "Homecare" },
                { id: 2, name: "Hospice" }
            ]);

            t.financialPeriodsCard = new ScoreCard({
                title: "Financial Periods",
                props: {
                    rowTemplate: 
                        "<div class='flex-field-container'>"+
                            "<div class='field-name field-name-largest'>${props.item.name}</div>"+
                            "<div class='field-value field-value-medium' style='text-align:right'>${props.item.value}</div>"+
                        "</div>"
                }
            });
            t.financialPeriodCardContainer.appendChild(t.financialPeriodsCard.domNode);

            t.addTableFilter("location",function(name,item,value) {
                var enabled = false;
                if (t.props.settings) {
                    enabled = true;
                    arrayUtils.some(t.props.settings.financialOrganizationalUnits,function(unit) {
                        if (unit.unit == item.id) {
                            enabled = (unit.enabledDate !== "");
                        }
                        return unit.unit == item.id;
                    });
                    if (enabled && t.props.settings.access) {
                        locationArray = t.props.settings.access.locations.toString().split(",");
                        enabled = t.props.settings.access.locations == "" || (arrayUtils.indexOf(locationArray,item.id) >= 0);
                    } else {
                        enabled = false;
                    }
                }
                return (item.active == "Yes") && (item.locationType != "") && enabled;
            });

            t.addTableFilter("type",function(name,item,value) {
                var enabled = true;
                if (t.props.settings) {
                    arrayUtils.some(t.props.settings.financialOrganizationalUnits,function(unit) {
                        if (unit.unit == item.id) {
                            enabled = (unit.enabledDate !== "");
                        }
                        return unit.unit == item.id;
                    });
                }
                return enabled;
            });

            t.addLogic(["type","location"],function(name,value) {
                if (!t.value || t.props.settings &&  t.value[(t.props.settings.financialOrganizationalUnitBasis == 1 ? "type" : "location")] != value) {
                    var financialPeriod = {};
                    financialPeriod[(t.props.settings.financialOrganizationalUnitBasis == 1 ? "type" : "location")] = value;
                    t.set("props",{ value : financialPeriod });
                    t.props.setParent("props",{ objectId: value });
                }
            });

            t.addRequirement(["type"],function() {
                return t.props.settings && t.props.settings.financialOrganizationalUnitBasis == 1;
            });
            t.addRequirement(["location"],function() {
                return t.props.settings && t.props.settings.financialOrganizationalUnitBasis == 2;
            });

            t.addDisabled(null,function(name,values) {
                if (!t.props.settings) {
                    return true;
                }
                var basis = t.props.settings.financialOrganizationalUnitBasis;
                if (basis == 1 && name != "type") {
                    return values["type"] == "";
                } else if (basis == 2 && name != "location") {
                    return values["location"] == "";
                }
            });
            t.addDisabled("date",true);
            t.addDisabled("addReportButton",function(name,values) {
                return values.report == "";
            });
        },

        _setValueAttr: function(value) {
            var t = this;
            t.inherited(arguments);
            if (value && value.organizationalUnit != t.id) {
                t.set("id",value.organizationalUnit);
            } else if (!value) {
                t.set("id",null);
            }
        },

        _setIdAttr: function(value) {
            var t = this;
            t.id = value;
            t._resetReports();
            if (t.value && t.value.reports) {
                arrayUtils.forEach(t.value.reports,function(report) {
                    t.loadReport(report);
                });
            }
        },

        refreshRendering: function() {
            var t = this;
            if (t.props.financialPeriods) {
                t.financialPeriodsCard.set({props: { data: t.props.financialPeriods } } );
            }
            if (t.props.settings) {
                var basis = t.props.settings.financialOrganizationalUnitBasis;
                domStyle.set(t.locationTypeContainer,   "display", basis == 1 ? "" : "none");
                domStyle.set(t.locationContainer,       "display", basis == 2 ? "" : "none");
            }
            if (t.props.financialReports) {
                t.setTable("FinancialReports",t.props.financialReports,true);
            }
            
            t.inherited(arguments);
        },

        _resetReports: function() {
            arrayUtils.forEach(this.reportItems,function(item) { item.destroy(); });
            this.reportItems = [];
        },

        _addReport: function() {
            var t = this;
            var reportTableItem = t.getTable("FinancialReports").get(t.getField("report").get("value"));
            reportTableItem.reportClass = reportTableItem.id;
            t.addReport(reportTableItem);
        },

        addReport: function(report) {
            var t = this;
            var reportItem = new ReportItem({
                props: {
                    label: report.name,
                    reportClass: report.reportClass,
                    onRemove: function(reportItem) {
                        t.reportItems = arrayUtils.filter(t.reportItems,function(item) { return item != reportItem });
                        reportItem.destroy();
                        arrayUtils.forEach(t.reportItems,function(item) {
                            item.reportNameField.validate();
                        });
                    },
                    onCopy: function(reportItemCopy) {
                        t.reportItemContainer.appendChild(reportItemCopy.domNode);
                        t.reportItems.push(reportItemCopy);
                    },
                    onValidateName: function(reportItem,name) {
                        return t.reportItems.length === 1 || arrayUtils.every(t.reportItems,function(item) {
                            return item === reportItem || item.reportNameField.get("value") != name;
                        });
                    },
                    onChangeName: function() {
                        arrayUtils.forEach(t.reportItems,function(item) {
                            item.reportNameField.validate();
                        });
                    },
                    onPushView: t.props.onPushView,
                    onGenerateReport: t.props.onGenerateReport,
                    financialPeriod: t.value
                }
            });
            t.reportItemContainer.appendChild(reportItem.domNode);
            t.reportItems.push(reportItem);
            t.setFormValue("report","",true);
            return reportItem;
        },

        loadReport: function(report) {
            var t = this;
            var reportItem = t.addReport(report);
            reportItem.setConfiguration(new PresetManager.PagePreset(report));
        },

        getSaveData: function() {
            var t = this;
            return {
                organizationalUnit: t.props.settings.financialOrganizationalUnitBasis == 1 ? t.value.type : t.value.location,
                reports: arrayUtils.map(t.reportItems,function(item) {
                    var configuration = item.reportView.controller.presetManager.currentPreset;
                    configuration.options.ReportClass = item.props.reportClass;
                    configuration.name = item.reportNameField.get("value");
                    arrayUtils.forEach(item.printOptionsView.reportControl.optionWidgets,function(widget) {
                        configuration.options[widget.optionDefinition.name] = widget.serialize();
                    });
                    return configuration;
                })
            }
        }
    });
});