define([
    "dojo/_base/declare",
    "thornberry/app/Component/_Component",
    "thornberry/app/Component/_EditorViewMixin",
    "dojo/_base/lang",
    "thornberry/app/Service/PatientInsurance",
    "thornberry/app/Component/PatientInsurance/PolicyEditor",
    "thornberry/app/Service/Settings"
],function(
    declare,
    _Component,
    _EditorViewMixin,
    lang,
    patientInsuranceService,
    PolicyEditor,
    settingsService
) {
    return declare([_Component,_EditorViewMixin], {
        // properties
        title: "Policy",
        helpPath: "/help/PatientInsurance.html#policy-editor",
        
        // _EditorContainerMixin props
        lockCheckEnabled: true,
        objectType: "Patient.Insurance.Insurance",
        EditorClass: PolicyEditor,

        // internals
        ndocNum: "",
        objectId: "",
        policy: null,
        sequenceData: null,
        patientInfo: null,
        billingComponentSettings: null,

        getInitialProps: function() {
            return lang.mixin(this.inherited(arguments),{
                ndocNum: "",
                objectId: ""
            });
        },

        getPageProps: function() {
            var t = this;
            return lang.mixin(t.inherited(arguments),{
                ndocNum: t.ndocNum
            });
        },

        refreshRendering: function() {
            var t = this;
            if (t.isShown) {
                t.set("ndocNum",t.props.ndocNum);
                t.set("objectId",t.props.objectId);

                if (!t.billingComponentSettings) {
                    t.addReadyPromise(settingsService.get("billingComponent").then(function(result) {
                        t.billingComponentSettings = result;
                        t.editor.set("props",{
                            billingComponentSettings: t.billingComponentSettings
                        });
                    }));
                }

                t.editor.set("props",{
                    value: t.policy,
                    policies: t.sequenceData,
                    mode: (t.props.objectId == "" ? "add" : "edit"),
                    patientType: (t.patientInfo ? t.patientInfo.patientType : null),
                    medicareNum: (t.patientInfo ? t.patientInfo.medicareNum : null),
                    medicaidNum: (t.patientInfo ? t.patientInfo.medicaidNum : null),
                    ndocNum: t.props.ndocNum,
                    disabled: (t.props.mode === "view")
                });
            }
            t.inherited(arguments);
        },

        _setNdocNumAttr: function(value) {
            var t = this;
            if (value && t.ndocNum !== value) {
                t.addReadyPromise(patientInsuranceService.getPolicies(value));
                t.setHandle("watchPolicies",patientInsuranceService.watchPolicies(value,lang.hitch(t,"updatePolicies")));

                t.addReadyPromise(patientInsuranceService.getGeneral(value));
                t.setHandle("watchGeneral",patientInsuranceService.watchGeneral(value,lang.hitch(t,"updateGeneral")));
            }
            t.ndocNum = value;
        },
        
        updatePolicies: function(data) {
            var t = this;
            var sequenceData = data.slice();
            sequenceData.unshift({ id: "NEW", insurance: "NEW POLICY", policyNum: "", effectiveDate: null, stopDate: null });
            t.sequenceData = sequenceData;
            t.editor.set("props",{ policies: sequenceData });
        },
        
        updateGeneral: function(data) {
            var t = this;
            data = data.length ? data[0] : {};
            t.patientInfo = data;
            t.editor.set("props",{
                "patientType" : data.patientType,
                "medicareNum" : data.medicareNum,
                "medicaidNum" : data.medicaidNum,
                "priorBillRatePolicies" : data.priorBillRatePolicies
            });
        },

        _setObjectIdAttr: function(value) {
            var t = this;
            if (value && t.objectId !== value) {
                t.objectId = value;

                t.addReadyPromise(patientInsuranceService.getPolicy(t.ndocNum,value).then(function(policy) {
                    if (t.objectId === value) {
                        t.set("policy",policy);
                    }
                }));
            } else if (!value) {
                t.set("policy",null);
            }
            t.objectId = value;
        },

        _setPolicyAttr: function(value) {
            var t = this;
            t.policy = value;
            t.editor.set("props",{
                value: value
            });
        },

        save: function() {
            var t = this;
            var data = lang.mixin(t.editor.get("value"),{
                ndocNum: t.props.ndocNum,
                id: t.props.objectId
            });
            delete data.billRates; // Only need to send modified/removed to the server
            return patientInsuranceService.savePolicy(data).then(function() {
                patientInsuranceService.getPolicySequences(t.props.ndocNum,true);
                patientInsuranceService.getPolicies(t.props.ndocNum,true);
                patientInsuranceService.getGeneral(t.props.ndocNum,true);
                t.objectId = null; // Force the editor to reload the object if the same object is edited immediately after
            });
        }
    });
});